@@ -1,13 +1,16 @@
 """Thin out the snapshots on disk."""
 
-from datetime import datetime
 import itertools
 import os
 import re
 import subprocess
+from datetime import datetime
+from typing import Pattern
 
-from snapintime.utils import config as config  # type: ignore
 import snapintime.utils.date  # type: ignore
+from snapintime.utils import config as config  # type: ignore
+
+from . import log
 
 
 def split_dir_hours(subvols: list, reg_ex) -> list:
@@ -17,14 +20,63 @@ def split_dir_hours(subvols: list, reg_ex) -> list:
     :param reg_ex: A re object defining the regular expression to evaluate against.
     :returns: A list that has only the items that passed the regular expression.
     """
+    return [subvol for subvol in subvols if reg_ex.search(subvol) is not None]
+
+
+def get_subvols_by_date(directory: str, reg_ex, remote: bool = False, remote_location: str = "") -> list:
+    """Return a list based on matching regular expression.
+
+    This is meant to produce the list that will be the input for one of the culling functions.
+
+    :param remote_location: This should be a string like user@computer or user@IPaddress
+    :param remote: True if this is taking place on the remote system
+    :param directory: The directory we want to grab subvols from.
+    :param reg_ex: A regular expression to apply to the directory contents.
+    :returns: A list of subvolumes for culling.
+    """
+    if remote and remote_location:
+        command = f"ssh {remote_location} ls {directory}"
+        results = subprocess.run(command, capture_output=True, shell=True, check=True, text=True)
+        subvols = results.stdout.split('\n')
+    else:
+        subvols = os.listdir(path=directory)
+    return [subvol for subvol in subvols if reg_ex.search(subvol) is not None]
+
+
+def btrfs_del(directory: str, subvols: list, remote: bool = False, remote_location: str = "") -> list[str]:
+    """Delete subvolumes in a given directory.
+
+    :param remote_location: This should be a string like user@computer or user@IPaddress
+    :param remote: If True, working on remote system
+    :param directory: The directory holding the subvolumes.
+    :param subvols: A list of subvolumes to delete
+    :returns: A list with the commands run and the results or, if there weren't any subvolumes\
+    to delete, returns a message with that information.
+    """
     return_list = []
-    for subvol in subvols:
-        if reg_ex.search(subvol) is not None:
-            return_list.append(subvol)
+    if subvols:
+        for subvol in subvols:
+            if remote and remote_location:
+                command = f"ssh {remote_location} /usr/sbin/btrfs sub del {directory}/{subvol}"
+            else:
+                command = f"/usr/sbin/btrfs sub del {directory}/{subvol}"
+            try:
+                log.debug(f'{remote=}')
+                log.debug(f'{remote_location=}')
+                log.debug(f'{command=}')
+                raw_result = subprocess.run(command, capture_output=True, shell=True, check=True, text=True)
+                return_text = f"Ran {raw_result.args} with a return code of {raw_result.returncode}.\n" \
+                              f"Result was {str(raw_result.stdout)}"
+                return_list.append(return_text)
+            except subprocess.SubprocessError as e:
+                error_text = f"Ran {e.args[1]} with a return code of {e.returncode}.\nResult was {str(e.stderr)}"  # type: ignore
+                return_list.append(error_text)
+    else:
+        return_list = ["There was either only one or no subvolumes at that date"]
     return return_list
 
 
-def daily_cull(dir_to_cull: list) -> list:
+def generate_daily_cull_list(dir_to_cull: list) -> list:
     """Take a list of snapshots from a directory (already reduced to one day) and cull.
 
     This culling will produce the closest it can to 4 snapshots\
@@ -42,73 +94,34 @@ def daily_cull(dir_to_cull: list) -> list:
     snapshots.
     :returns: A list containing all the subvolumes to cull.
     """
-    fourths = [re.compile('[0][0-5][0-9][0-9]$'), re.compile('[0][6-9][0-9][0-9]$|[1][0-1][0-9][0-9]$'),
-               re.compile('[1][2-7][0-9][0-9]$'), re.compile('[1][8-9][0-9][0-9]$|[2][0-3][0-9][0-9]$')]
-    fourths_list = []
-    for fourth in fourths:
-        fourths_list.append(split_dir_hours(dir_to_cull, fourth)[1:])
+    fourths: list[Pattern[str]] = [re.compile(r'0[0-5]\d\d$'),
+                                   re.compile(r'0[6-9]\d\d$|1[0-1]\d\d$'),
+                                   re.compile(r'1[2-7]\d\d$'),
+                                   re.compile(r'1[8-9]\d\d$|2[0-3]\d\d$')]
+    fourths_list = [split_dir_hours(dir_to_cull, fourth)[1:] for fourth in fourths]
     return list(itertools.chain.from_iterable(fourths_list))
 
 
-def get_subvols_by_date(directory: str, reg_ex) -> list:
-    """Return a list based on matching regular expression.
-
-    This is meant to produce the list that will be the input for one of the culling functions.
-
-    :param directory: The directory we want to grab subvols from.
-    :param reg_ex: A regular expression to apply to the directory contents.
-    :returns: A list of subvolumes for culling.
-    """
-    subvols = os.listdir(path=directory)
-    return_list = []
-    for subvol in subvols:
-        if reg_ex.search(subvol) is not None:
-            return_list.append(subvol)
-    return return_list
-
-
-def btrfs_del(directory: str, subvols: list) -> list:
-    """Delete subvolumes in a given directory.
-
-    :param directory: The directory holding the subvolumes.
-    :param subvols: A list of subvolumes to delete
-    :returns: A list with the commands run and the results or, if there weren't any subvolumes\
-    to delete, returns a message with that information.
-    """
-    return_list = []
-    if len(subvols) > 0:
-        for subvol in subvols:
-            command = f"/usr/sbin/btrfs sub del {directory}/{subvol}"
-            try:
-                raw_result = subprocess.run(command, capture_output=True, shell=True, check=True, text=True)
-                return_text = f"Ran {raw_result.args} with a return code of {raw_result.returncode}.\n"\
-                    f"Result was {str(raw_result.stdout)}"
-                return_list.append(return_text)
-            except subprocess.SubprocessError as e:
-                error_text = f"Ran {e.args[1]} with a return code of {e.returncode}.\nResult was {str(e.stderr)}"  # type: ignore
-                return_list.append(error_text)
-    else:
-        return_list = ["There was either only one or no subvolumes at that date"]
-    return return_list
-
-
-def cull_three_days_ago(config: dict) -> list:
+def cull_three_days_ago(configuration: dict) -> list:
     """Cull the btrfs snapshots from 3 days ago.
 
-    :param config: The configuration file.
+    Take snapshots that were taken on the third day in the past and cull to 4 (max) snapshots.
+
+    :param configuration: The configuration file.
     :returns: A list containing the results of running the commands.
     """
+    location: str = "backuplocation"
     three_days_ago: str = snapintime.utils.date.prior_date(datetime.now(), 3).strftime("%Y-%m-%d")
     three_days_ago_reg_ex = re.compile(three_days_ago)
     return_list = []
-    for subvol in config.values():
-        subvols_three_days_ago = get_subvols_by_date(subvol.get("backuplocation"), three_days_ago_reg_ex)
-        three_days_ago_culled = daily_cull(subvols_three_days_ago)
-        return_list.append(btrfs_del(subvol.get("backuplocation"), three_days_ago_culled))
+    for subvol in configuration.values():
+        subvols_three_days_ago = get_subvols_by_date(subvol.get(location), three_days_ago_reg_ex)
+        three_days_ago_culled = generate_daily_cull_list(subvols_three_days_ago)
+        return_list.append(btrfs_del(subvol.get(location), three_days_ago_culled))
     return return_list
 
 
-def weekly_cull(dir_to_cull: list) -> list:
+def generate_weekly_cull_list(dir_to_cull: list) -> list:
     """Take a list of snapshots from a directory (already reduced to one day) and cull.
 
         This culling will return a list with the snapshots to remove for the given day.
@@ -119,7 +132,7 @@ def weekly_cull(dir_to_cull: list) -> list:
         - day1-1800
 
         :param dir_to_cull: A list containing snapshots. Assumes another function\
-        has already reduced this list to a list containing only one day's worth of\
+        has already reduced this list to a list containing only one week's worth of\
         snapshots.
         :returns: A list containing all the subvolumes to cull.
         """
@@ -129,23 +142,30 @@ def weekly_cull(dir_to_cull: list) -> list:
     return sorted_dir_to_cull
 
 
-def cull_seven_days_ago(config: dict) -> list:
+def cull_seven_days_ago(configuration: dict, remote: bool = False) -> list:
     """Cull the btrfs snapshots from 7 days ago.
 
-    :param config: The configuration file.
+    :param remote: True if culling on the remote server
+    :param configuration: The configuration file.
     :returns: A list containing the results of running the commands.
     """
     seven_days_ago: str = snapintime.utils.date.prior_date(datetime.now(), 7).strftime("%Y-%m-%d")
     seven_days_ago_reg_ex = re.compile(seven_days_ago)
     return_list = []
-    for subvol in config.values():
-        subvols_seven_days_ago = get_subvols_by_date(subvol.get("backuplocation"), seven_days_ago_reg_ex)
-        seven_days_ago_culled = weekly_cull(subvols_seven_days_ago)
-        return_list.append(btrfs_del(subvol.get("backuplocation"), seven_days_ago_culled))
+    for subvol in configuration.values():
+        location: str = "remote_subvol_dir" if remote else "backuplocation"
+        subvols_seven_days_ago = get_subvols_by_date(subvol.get(location), seven_days_ago_reg_ex)
+        if remote:
+            subvols_seven_days_ago = remove_protected(subvol, subvols_seven_days_ago)
+        if len(subvols_seven_days_ago) != 0:
+            seven_days_ago_culled = generate_weekly_cull_list(subvols_seven_days_ago)
+            return_list.append(btrfs_del(subvol.get(location), seven_days_ago_culled, remote,
+                               remote_location=subvol.get('remote_location')))
+
     return return_list
 
 
-def quarterly_yearly_cull(dir_to_cull: list) -> list:
+def generate_quarterly_yearly_cull_list(dir_to_cull: list) -> list:
     """Take a list of snapshots from a directory (already reduced to one week or quarter) and cull.
 
             This culling will return a list with the snapshots to remove for the week or quarter.
@@ -168,58 +188,75 @@ def quarterly_yearly_cull(dir_to_cull: list) -> list:
     return sorted_dir_to_cull
 
 
-def cull_last_quarter(config: dict) -> list:
+def remove_protected(subvol: dict, subvol_list_to_pare: list):
+    protected_snapshots = subvol.get("remote_protected")
+    if protected_snapshots is None:
+        protected_snapshots = []
+    return [subvol for subvol in subvol_list_to_pare if subvol not in protected_snapshots]
+
+
+def cull_last_quarter(configuration: dict, remote: bool = False) -> list:
     """Cull the btrfs snapshots from quarter.
 
     Should leave 1 snapshot per week for 13 weeks.
 
-    :param config: The configuration file.
+    :param remote: Are we doing this on the remote system?
+    :param configuration: The configuration file.
     :returns: A list containing the results of running the commands.
     """
     last_quarter: list = snapintime.utils.date.quarterly_weeks(datetime.now())
+    location: str = "remote_subvol_dir" if remote else "backuplocation"
     return_list = []
-    for subvol in config.values():
+    for subvol in configuration.values():
         for week in last_quarter:
             reg_ex_string = ""
             for day in week:
-                reg_ex_string = reg_ex_string + f'({day.strftime("%Y-%m-%d")})|'
+                reg_ex_string = f"{reg_ex_string}({day.strftime('%Y-%m-%d')})|"
             reg_ex_string_minus_final_or = reg_ex_string[:-1]
             weekly_reg_ex = re.compile(reg_ex_string_minus_final_or)
-            subvols_this_week = get_subvols_by_date(subvol.get("backuplocation"), weekly_reg_ex)
+            subvols_this_week = get_subvols_by_date(subvol.get(location), weekly_reg_ex,
+                                                    remote, subvol.get("remote_location"))
+            if remote:
+                subvols_this_week = remove_protected(subvol, subvols_this_week)
             if len(subvols_this_week) != 0:
-                this_week_culled = quarterly_yearly_cull(subvols_this_week)
-                return_list.append(btrfs_del(subvol.get("backuplocation"), this_week_culled))
+                this_week_culled = generate_quarterly_yearly_cull_list(subvols_this_week)
+                return_list.append(btrfs_del(subvol.get(location), this_week_culled, remote,
+                                             remote_location=subvol.get('remote_location')))
     return return_list
 
 
-def cull_last_year(config: dict) -> list:
+def cull_last_year(configuration: dict, remote: bool = False) -> list:
     """Cull the btrfs snapshots from quarter.
 
-    Should leave 1 snapshot per week for 13 weeks.
+    Should leave 1 snapshot per quarter for 4 quarters.
 
-    :param config: The configuration file.
+    :param remote: If true, we're doing this on the remote system.
+    :param configuration: The configuration file.
     :returns: A list containing the results of running the commands.
     """
     last_year: list = snapintime.utils.date.yearly_quarters(datetime.now())
     return_list = []
-    for subvol in config.values():
+    for subvol in configuration.values():
         for quarter in last_year:
             reg_ex_string = ""
             for day in quarter:
-                reg_ex_string = reg_ex_string + f'({day.strftime("%Y-%m-%d")})|'
+                reg_ex_string = f"{reg_ex_string}({day.strftime('%Y-%m-%d')})|"
             reg_ex_string_minus_final_or = reg_ex_string[:-1]
             quarterly_reg_ex = re.compile(reg_ex_string_minus_final_or)
             subvols_this_quarter = get_subvols_by_date(subvol.get("backuplocation"), quarterly_reg_ex)
+            if remote:
+                subvols_this_quarter = remove_protected(subvol, subvols_this_quarter)
             if len(subvols_this_quarter) != 0:
-                this_quarter_culled = quarterly_yearly_cull(subvols_this_quarter)
-                return_list.append(btrfs_del(subvol.get("backuplocation"), this_quarter_culled))
+                this_quarter_culled = generate_quarterly_yearly_cull_list(subvols_this_quarter)
+                return_list.append(btrfs_del(subvol.get("backuplocation"), this_quarter_culled, remote,
+                                             remote_location=subvol.get('remote_location')))
     return return_list
 
 
 def print_output(list_of_lists: list):  # pragma: no cover
     for directory in list_of_lists:
         for result in directory:
-            print(result)
+            log.info(result)
 
 
 def main():  # pragma: no cover
